'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SchemaItem = exports.SchemaEntity = undefined;

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _symbol = require('babel-runtime/core-js/symbol');

var _symbol2 = _interopRequireDefault(_symbol);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _actions = require('./actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var singleton = (0, _symbol2.default)();
var singletonEnforcer = (0, _symbol2.default)();

var SchemaError = function (_Error) {
  (0, _inherits3.default)(SchemaError, _Error);

  function SchemaError(message) {
    (0, _classCallCheck3.default)(this, SchemaError);

    var _this = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(SchemaError).call(this, message));

    _this.name = _this.constructor.name;
    _this.message = message;
    if (typeof Error.captureStackTrace === 'function') {
      Error.captureStackTrace(_this, _this.constructor);
    } else {
      _this.stack = new Error(message).stack;
    }
    return _this;
  }

  return SchemaError;
}(Error);

// The schema handler. Should handle all entities and related actions.


var Schema = function () {
  function Schema(enforcer) {
    (0, _classCallCheck3.default)(this, Schema);

    if (enforcer != singletonEnforcer) throw new SchemaError('Cannot call Schema directly. Use Schema.instance instead.');
  }

  // Is a singleton still necessary?


  (0, _createClass3.default)(Schema, [{
    key: 'createItem',


    // Creates an empty item.
    value: function createItem(type, attributes, relationships) {
      if (!this.entities) throw new SchemaError('The schema was not initialized properly. Run initialize() first.');
      if (!this.entities[type]) throw new SchemaError(type + ' is not a valid entity.');else {
        return new SchemaItem({
          id: null,
          type: type,
          attributes: attributes,
          relationships: relationships || {}
        });
      }
    }

    // Creates the base structure of the entity.

  }, {
    key: 'initialize',
    value: function initialize(store, entities) {
      this.store = store;
      if (!entities || (typeof entities === 'undefined' ? 'undefined' : (0, _typeof3.default)(entities)) !== 'object' || !entities.length) throw new SchemaError('Schema should receive a list of entity names as a parameter.');
      this.entities = {};
      var state = store.getState();
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(entities), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var type = _step.value;

          var entity = new SchemaEntity(type);
          this.entities[type] = entity;
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }

    // Allows to call an action related to an entity or an element.

  }, {
    key: 'dispatch',
    value: function dispatch(actionCreator) {
      return this.store.dispatch(actionCreator);
    }
  }], [{
    key: 'fromArray',


    // Creates a subset of SchemaEntity with items specified by their IDs.
    // Should be compatible with all SchemaEntity operations.
    value: function fromArray(type, ids) {
      if (!ids instanceof Array) throw new SchemaError('Schema.fromArray() should receive a type and an array of IDs.');
      return new SchemaEntity(type, ids);
    }
  }, {
    key: 'instance',
    get: function get() {
      if (!this[singleton]) {
        this[singleton] = new Schema(singletonEnforcer);
      }
      return this[singleton];
    }
  }]);
  return Schema;
}();

// Defines a container of SchemaItem elements of the same type.


var SchemaEntity = exports.SchemaEntity = function () {
  function SchemaEntity(type) {
    var ids = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];
    (0, _classCallCheck3.default)(this, SchemaEntity);

    this.type = type;
    this.ids = ids;
  }

  // Returns all items from the schema as raw objects.
  // Note that it filters all items that weren't found on the state.
  // TODO: fetch non found items from DB?


  (0, _createClass3.default)(SchemaEntity, [{
    key: 'findById',


    // gets an element from an entity from its ID.
    // returns null if no results are found.
    value: function findById(id) {
      var result = this.items.filter(function (item) {
        return item.id == id;
      });
      return result && result.length ? new SchemaItem(result[0]) : null;
    }

    // performs a simple search on an entity.
    // TODO: full comparator support (eq, ne, like)

  }, {
    key: 'where',
    value: function where(params) {
      return _lodash2.default.filter(this.items, function (item) {
        return _lodash2.default.reduce(params, function (result, value, key) {
          return result && (key == 'id' ? item : item.attributes)[key] == value;
        }, true);
      }).map(function (item) {
        return new SchemaItem(item);
      });
    }
  }, {
    key: 'items',
    get: function get() {
      var entity = Schema.instance.store.getState().api[this.type] || { data: [] };
      if (this.ids.length) {
        return this.ids.map(function (id) {
          return _lodash2.default.find(entity.data, { id: id });
        }).filter(function (item) {
          return item;
        });
      } else {
        return entity.data;
      }
    }

    // returns all items from the schema as SchemaItem objects.

  }, {
    key: 'all',
    get: function get() {
      return this.items.map(function (item) {
        return new SchemaItem(item);
      });
    }

    // returns the first result in a SchemaEntity.

  }, {
    key: 'first',
    get: function get() {
      return this.items.length ? new SchemaItem(this.items[0]) : undefined;
    }

    // returns the number of items in a SchemaEntity.

  }, {
    key: 'count',
    get: function get() {
      return this.items.length;
    }
  }]);
  return SchemaEntity;
}();

// Defines a single item.


var SchemaItem = exports.SchemaItem = function () {
  function SchemaItem(item) {
    (0, _classCallCheck3.default)(this, SchemaItem);

    this.id = item.id;
    this.type = item.type;
    this.diff = (0, _assign2.default)({}, {
      attributes: item.attributes || {},
      relationships: item.relationships || {}
    });
    if (!item.id) this._ref = {
      attributes: item.attributes || {},
      relationships: item.relationships || {}
    };
  }

  // Returns the raw item from the state or a temporary store.


  (0, _createClass3.default)(SchemaItem, [{
    key: 'sanitize',


    // Returns a sanitized item, ready to be sent through a JSON API query.
    value: function sanitize() {
      return {
        id: this.id ? this.id.toString() : undefined,
        type: this.type,
        attributes: this.diff.attributes || {},
        relationships: this.diff.relationships ? _lodash2.default.mapValues(this.diff.relationships, function (rel) {
          return {
            data: rel.data
          };
        }) : {}
      };
    }

    // Returns an array of all SchemaItem elements of a given type related to
    // the item.

  }, {
    key: 'getRelated',
    value: function getRelated(relation) {
      var related = this.relationships[relation];
      if (related && related.data && related.data.length) {
        var _ret = function () {
          var entity = Schema.instance.entities[related.data[0].type].items;
          if (entity.length) {
            var result = related.data.map(function (item) {
              return new SchemaItem(_lodash2.default.find(entity, function (res) {
                return res.id == item.id;
              }));
            });
            return {
              v: result[0] ? result : []
            };
          }
        }();

        if ((typeof _ret === 'undefined' ? 'undefined' : (0, _typeof3.default)(_ret)) === "object") return _ret.v;
      }
      return [];
    }

    // Counts all related items of a specific type.

  }, {
    key: 'countRelated',
    value: function countRelated(relation) {
      var related = this.relationships[relation];
      return related && related.data && related.data.length ? related.data.length : 0;
    }
  }, {
    key: 'update',
    value: function update(data) {
      // Only attributes and relationships should be updated, everything else is
      // treated as static. Changes are saved on a temporary `diff` list.
      this.diff = _lodash2.default.merge({}, this.diff, data);
      this.attributes = _lodash2.default.merge({}, this.attributes, data.attributes || {});
      this.relationships = _lodash2.default.merge({}, this.relationships, data.relationships || {});
    }

    // Triggers a CreateAction event for this item.

  }, {
    key: 'save',
    value: function save() {
      var fn = this.id ? _actions.update : _actions.create;
      return fn(this.sanitize()); // TODO: clean the diff object
    }
  }, {
    key: 'ref',
    get: function get() {
      var _this2 = this;

      if (this.id) {
        // it's an existing item
        var entity = Schema.instance.store.getState().api[this.type],
            result = void 0;
        if (entity) {
          result = _lodash2.default.filter(entity.data, function (item) {
            return item.id === _this2.id;
          });
        }
        return result ? result[0] : null;
      } else {
        // it's a new item
        return this._ref;
      }
    },
    set: function set(data) {
      if (!this.id) this._ref = _lodash2.default.merge({}, this._ref, data);
    }

    // Returns the attributes object from the array.
    // TODO: The ID should be gathered from here too.

  }, {
    key: 'attributes',
    get: function get() {
      return this.ref ? this.ref.attributes : {};
    },
    set: function set(data) {
      this.ref.attributes = data;
    }

    // Returns the raw list of relationships from the state.

  }, {
    key: 'relationships',
    get: function get() {
      return this.ref ? this.ref.relationships : [];
    },
    set: function set(data) {
      this.ref.relationships = data;
    }
  }]);
  return SchemaItem;
}();

exports.default = Schema;